/** @file
 <<The file is used to create, update Data Hub Record entries in a MSA file.>>
 
 <<The DataHbsDlg is a dialog called to add or edit a Data Hub Record within a MSA file.>>
 
 Copyright (c) 2006, Intel Corporation
 All rights reserved. This program and the accompanying materials
 are licensed and made available under the terms and conditions of the BSD License
 which accompanies this distribution.  The full text of the license may be found at
 http://opensource.org/licenses/bsd-license.php
 
 THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
 WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
 
 Package Name: Tools
 Module Name:  FrameworkWizard
 
 **/

package org.tianocore.frameworkwizard.module.ui.dialog;

import java.awt.event.ActionEvent;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextArea;

import org.tianocore.frameworkwizard.common.DataType;
import org.tianocore.frameworkwizard.common.DataValidation;
import org.tianocore.frameworkwizard.common.EnumerationData;
import org.tianocore.frameworkwizard.common.Log;
import org.tianocore.frameworkwizard.common.Tools;
import org.tianocore.frameworkwizard.common.ui.ArchCheckBox;
import org.tianocore.frameworkwizard.common.ui.IDialog;
import org.tianocore.frameworkwizard.common.ui.IFrame;
import org.tianocore.frameworkwizard.common.ui.StarLabel;
import org.tianocore.frameworkwizard.module.Identifications.DataHubs.DataHubsIdentification;

/**
 * The class is used to create, update DataHub of MSA/MBD file It extends
 * IInternalFrame
 * 
 * 
 * 
 */
public class DataHubsDlg extends IDialog {

  // /
  // / Define class Serial Version UID
  // /
  private static final long serialVersionUID = -3667906991966638892L;

  //
  // Define class members
  //
  private JPanel jContentPane = null;

  private JLabel jLabelUsage = null;

  private JComboBox jComboBoxUsage = null;

  private JLabel jLabelDataHubRecord = null;

  private JTextField jTextFieldDataHubRecord = null;

  private StarLabel jStarLabel1 = null;

  private StarLabel jStarLabel2 = null;

  private JLabel jLabelFeatureFlag = null;

  private JTextField jTextFieldFeatureFlag = null;

  private JLabel jLabelArch = null;

  private JScrollPane jScrollPane = null;

  private JLabel jLabelHelpText = null;

  private JTextArea jTextAreaHelpText = null;

  private JScrollPane jScrollPaneHelpText = null;

  private ArchCheckBox jArchCheckBox = null;

  private JButton jButtonOk = null;

  private JButton jButtonCancel = null;

  //
  // Not used by UI
  //
  private DataHubsIdentification id = null;

  private EnumerationData ed = new EnumerationData();

  /**
   * This method initializes jTextFieldDataHubRecord
   * 
   * @return javax.swing.JTextField jTextFieldDataHubRecord
   * 
   */
  private JTextField getJTextFieldDataHubRecord() {
    if (jTextFieldDataHubRecord == null) {
      jTextFieldDataHubRecord = new JTextField();
      jTextFieldDataHubRecord
          .setBounds(new java.awt.Rectangle(160, 10, 320, 20));
      jTextFieldDataHubRecord.setPreferredSize(new java.awt.Dimension(320, 20));
      jTextFieldDataHubRecord
          .setToolTipText("Enter the C Name of the Data Hub Record");
    }
    return jTextFieldDataHubRecord;
  }

  /**
   * This method initializes jComboBoxUsage
   * 
   * @return javax.swing.JComboBox jComboBoxUsage
   * 
   */
  private JComboBox getJComboBoxUsage() {
    if (jComboBoxUsage == null) {
      jComboBoxUsage = new JComboBox();
      jComboBoxUsage.setBounds(new java.awt.Rectangle(160, 35, 320, 20));
      jComboBoxUsage.setPreferredSize(new java.awt.Dimension(320, 20));
      jComboBoxUsage
          .setToolTipText("<html><table><tr><td>ALWAYS_CONSUMED</td><td>Module always consumes a Data Hub Entry<br>via registering a filter driver.</td></tr><tr><td>SOMETIMES_CONSUMED</td><td>Module will use a Data Hub Entry if it exists<br>via registering a filter driver</td></tr><tr><td>ALWAYS_PRODUCED</td><td>Always logs data into the Data Hub</td></tr><tr><td>SOMETIMES_PRODUCED</td><td>Log data into the Data Hub under<br>certain circumstances</td></tr></table></html>");
    }
    return jComboBoxUsage;
  }

  /**
   * This method initializes jTextFieldFeatureFlag
   * 
   * @return javax.swing.JTextField
   */
  private JTextField getJTextFieldFeatureFlag() {
    if (jTextFieldFeatureFlag == null) {
      jTextFieldFeatureFlag = new JTextField();
      jTextFieldFeatureFlag.setBounds(new java.awt.Rectangle(160, 105, 320, 20));
      jTextFieldFeatureFlag.setPreferredSize(new java.awt.Dimension(320, 20));
      jTextFieldFeatureFlag.setToolTipText("Postfix expression that must evaluate to TRUE or FALSE");
    }
    return jTextFieldFeatureFlag;
  }

  /**
   * This method initializes jScrollPane
   * 
   * @return javax.swing.JScrollPane
   */
  private JScrollPane getJScrollPane() {
    if (jScrollPane == null) {
      jScrollPane = new JScrollPane();
      jScrollPane.setViewportView(getJContentPane());
    }
    return jScrollPane;
  }

  /**
   * This method initializes jTextAreaHelpText
   * 
   * @return javax.swing.JTextArea
   * 
   */
  private JTextArea getJTextAreaHelpText() {
    if (jTextAreaHelpText == null) {
      jTextAreaHelpText = new JTextArea();
      jTextAreaHelpText.setLineWrap(true);
      jTextAreaHelpText.setWrapStyleWord(true);
    }
    return jTextAreaHelpText;
  }

  /**
   * This method initializes jScrollPaneHelpText
   * 
   * @return javax.swing.JScrollPane
   * 
   */
  private JScrollPane getJScrollPaneHelpText() {
    if (jScrollPaneHelpText == null) {
      jScrollPaneHelpText = new JScrollPane();
      jScrollPaneHelpText.setHorizontalScrollBarPolicy(javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
      jScrollPaneHelpText.setSize(new java.awt.Dimension(320, 40));
      jScrollPaneHelpText.setPreferredSize(new java.awt.Dimension(320,40));
      jScrollPaneHelpText.setLocation(new java.awt.Point(160,60));
      jScrollPaneHelpText.setViewportView(getJTextAreaHelpText());
    }
    return jScrollPaneHelpText;
  }
  /**
   * This method initializes jButtonOk
   * 
   * @return javax.swing.JButton
   * 
   */
  private JButton getJButtonOk() {
    if (jButtonOk == null) {
      jButtonOk = new JButton();
      jButtonOk.setBounds(new java.awt.Rectangle(290, 157, 90, 20));
      jButtonOk.setText("Ok");
      jButtonOk.addActionListener(this);
    }
    return jButtonOk;
  }

  /**
   * This method initializes jButtonCancel
   * 
   * @return javax.swing.JButton
   * 
   */
  private JButton getJButtonCancel() {
    if (jButtonCancel == null) {
      jButtonCancel = new JButton();
      jButtonCancel.setBounds(new java.awt.Rectangle(390, 157, 90, 20));
      jButtonCancel.setText("Cancel");
      jButtonCancel.addActionListener(this);
    }
    return jButtonCancel;
  }

  public static void main(String[] args) {

  }

  /**
   * This method initializes this
   * 
   */
  private void init() {
    this.setSize(500, 230);
    this.setContentPane(getJScrollPane());
    this.setTitle("Data Hubs");
    initFrame();
    this.setViewMode(false);
    this.centerWindow();
  }

  /**
   * This method initializes this Fill values to all fields if these values are
   * not empty
   * 
   * @param inDataHubsId
   * 
   */
  private void init(DataHubsIdentification inDataHubsId) {
    init();
    this.id = inDataHubsId;

    if (this.id != null) {
      this.jTextFieldDataHubRecord.setText(id.getName());
      this.jComboBoxUsage.setSelectedItem(id.getUsage());
      this.jTextAreaHelpText.setText(id.getHelp());
      this.jTextFieldFeatureFlag.setText(id.getFeatureFlag());
      this.jArchCheckBox.setSelectedItems(id.getSupArchList());
    }
  }

  /**
   * This is the override edit constructor
   * 
   * @param inDataHubsIdentification
   * @param iFrame
   * 
   */
  public DataHubsDlg(DataHubsIdentification inDataHubsIdentification,
      IFrame iFrame) {
    super(iFrame, true);
    init(inDataHubsIdentification);
  }

  /**
   * Disable all components when the mode is view
   * 
   * @param isView
   *          true - The view mode; false - The non-view mode
   * 
   */
  public void setViewMode(boolean isView) {
    if (isView) {
      this.jTextFieldDataHubRecord.setEnabled(!isView);
      this.jComboBoxUsage.setEnabled(!isView);
    }
  }

  /**
   * This method initializes jContentPane
   * 
   * @return javax.swing.JPanel jContentPane
   * 
   */
  private JPanel getJContentPane() {
    if (jContentPane == null) {
      jStarLabel1 = new StarLabel();
      jStarLabel1.setLocation(new java.awt.Point(2, 10));
      jLabelDataHubRecord = new JLabel();
      jLabelDataHubRecord.setBounds(new java.awt.Rectangle(15, 10, 145, 20));
      jLabelDataHubRecord.setText("Data Hub Record");
      jStarLabel2 = new StarLabel();
      jStarLabel2.setLocation(new java.awt.Point(2, 35));
      jLabelUsage = new JLabel();
      jLabelUsage.setText("Usage");
      jLabelUsage.setBounds(new java.awt.Rectangle(15, 35, 145, 20));
      jLabelHelpText = new JLabel();
      jLabelHelpText.setBounds(new java.awt.Rectangle(14, 60, 145, 20));
      jLabelHelpText.setText("Help Text");
      jLabelFeatureFlag = new JLabel();
      jLabelFeatureFlag.setBounds(new java.awt.Rectangle(15, 105, 145, 20));
      jLabelFeatureFlag.setText("Feature Flag Expression");
      jLabelArch = new JLabel();
      jLabelArch.setBounds(new java.awt.Rectangle(15, 130, 145, 20));
      jLabelArch.setText("Supported Architectures");
      jArchCheckBox = new ArchCheckBox();
      jArchCheckBox.setBounds(new java.awt.Rectangle(160, 130, 320, 20));
      jArchCheckBox.setPreferredSize(new java.awt.Dimension(320, 20));

      jContentPane = new JPanel();
      jContentPane.setLayout(null);
      jContentPane.setPreferredSize(new java.awt.Dimension(485, 185));

      jContentPane.add(jLabelDataHubRecord, null);
      jContentPane.add(jLabelUsage, null);
      jContentPane.add(jLabelFeatureFlag, null);
      jContentPane.add(jLabelArch, null);

      jContentPane.add(getJTextFieldDataHubRecord(), null);
      jContentPane.add(getJComboBoxUsage(), null);
      jContentPane.add(getJTextFieldFeatureFlag(), null);

      jContentPane.add(jStarLabel1, null);
      jContentPane.add(jStarLabel2, null);

      jContentPane.add(jLabelHelpText, null);
      jContentPane.add(getJScrollPaneHelpText(), null);
      jContentPane.add(jArchCheckBox, null);
      jContentPane.add(getJButtonOk(), null);
      jContentPane.add(getJButtonCancel(), null);
    }
    return jContentPane;
  }

  /**
   * This method initializes Usage type
   * 
   */
  private void initFrame() {
    Tools.generateComboBoxByVector(jComboBoxUsage, ed.getVDataHubUsage());
  }

  /*
   * (non-Javadoc)
   * 
   * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
   * 
   * Override actionPerformed to listen all actions
   * 
   */
  public void actionPerformed(ActionEvent arg0) {
    if (arg0.getSource() == jButtonOk) {
      if (checkAdd()) {
        getCurrentDataHubs();
        this.returnType = DataType.RETURN_TYPE_OK;
        this.setVisible(false);
      }
    }

    if (arg0.getSource() == jButtonCancel) {
      this.returnType = DataType.RETURN_TYPE_CANCEL;
      this.setVisible(false);
    }
  }

  /**
   * Data validation for all fields
   * 
   * @retval true - All datas are valid
   * @retval false - At least one data is invalid
   * 
   */
  public boolean checkAdd() {
    //
    // Check if all fields have correct data types
    //

    //
    // Check DataHubRecord
    //
    if (isEmpty(this.jTextFieldDataHubRecord.getText())) {
      Log.wrn("Update Hubs", "Data Hub Record must not be empty");
      return false;
    }

    if (!isEmpty(this.jTextFieldDataHubRecord.getText())) {
      if (!DataValidation.isC_NameType(this.jTextFieldDataHubRecord.getText())) {
        Log.wrn("Update Hubs", "Incorrect data type for Data Hub Record");
        return false;
      }
    }

    //
    // Check FeatureFlag
    //
    if (!isEmpty(this.jTextFieldFeatureFlag.getText())) {
      if (!DataValidation.isFeatureFlag(this.jTextFieldFeatureFlag.getText())) {
        Log.wrn("Update Hubs", "Incorrect data type for Feature Flag");
        return false;
      }
    }

    return true;
  }

  private DataHubsIdentification getCurrentDataHubs() {
    String arg0 = this.jTextFieldDataHubRecord.getText();
    String arg1 = this.jComboBoxUsage.getSelectedItem().toString();

    String arg2 = this.jTextFieldFeatureFlag.getText();
    Vector<String> arg3 = this.jArchCheckBox.getSelectedItemsVector();
    String arg4 = this.jTextAreaHelpText.getText();

    id = new DataHubsIdentification(arg0, arg1, arg2, arg3, arg4);
    return id;
  }
  
  public DataHubsIdentification getId() {
    return id;
  }

  public void setId(DataHubsIdentification id) {
    this.id = id;
  }
}
